/* knocker version 0.6.0
 * Release date: 17 February 2002
 *
 * Project homepage: http://knocker.sourceforge.net
 *
 * Copyright 2001,2002 Gabriele Giorgetti <g.gabriele79@genie.it>
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "knocker_core.h"
#include "knocker_args.h"
#include "knocker_log.h"


/*
   ============================================================================
   ============================================================================
*/
static void print_version_info (void)
{
  fprintf (stdout, "%s %s (%s)\n", PACKAGE, VERSION, knocker_core_release_date);
  fprintf (stdout, "\n");
  fprintf (stdout, "Written by %s %s\n", knocker_core_author, knocker_core_author_email );
  fprintf (stdout, "Copyright (C) 2001,2002 %s\n", knocker_core_author);
  fprintf (stdout, "\n");
  fprintf (stdout, "This software is released under the GNU GPL\n");
  fprintf (stdout, "\n");
  fprintf (stdout, "%s can be found at %s\n", PACKAGE, knocker_core_homepage);
}


/*
   ============================================================================
   ============================================================================
*/
static void print_help_info (void)
{
  fprintf (stdout, "%s, the net portscanner. Version %s (%s)\n", PACKAGE, VERSION, knocker_core_release_date);
  fprintf (stdout, "\n");
  fprintf (stdout, "Usage: %s %s <HOST> \n", PACKAGE, HOST_LONG_OPT);
  fprintf (stdout, "   or: %s %s <HOST> %s <PORT> %s <PORT> \n", PACKAGE, HOST_LONG_OPT, START_PORT_LONG_OPT, END_PORT_LONG_OPT);
  fprintf (stdout, "   or: %s %s <HOST> %s <PORT>\n", PACKAGE, HOST_LONG_OPT, SINGLE_PORT_LONG_OPT);
  fprintf (stdout, "   or: %s %s <HOST> %s <PORT> %s <PORT> %s\n", PACKAGE, HOST_SHORT_OPT, START_PORT_SHORT_OPT, END_PORT_SHORT_OPT, QUIET_MODE_LONG_OPT);
  fprintf (stdout, "\n");
  fprintf (stdout, "Example: %s %s 192.168.0.1 %s 1 %s 1024\n", PACKAGE, HOST_SHORT_OPT, START_PORT_SHORT_OPT, END_PORT_SHORT_OPT);
  fprintf (stdout, "\n");
  fprintf (stdout, "Required options:\n");
  fprintf (stdout, "      %s,  %s             host name or numeric Internet address\n", HOST_SHORT_OPT, HOST_LONG_OPT);
  fprintf (stdout, "\n");
  fprintf (stdout, "Common options (if SP is specified you must also give EP):\n");
  fprintf (stdout, "      %s,  %s             single port number (for one port scans only)\n", SINGLE_PORT_SHORT_OPT, SINGLE_PORT_LONG_OPT);
  fprintf (stdout, "      %s, %s       port number to begin the scan from\n", START_PORT_SHORT_OPT, START_PORT_LONG_OPT);
  fprintf (stdout, "      %s, %s         port number to end the scan at\n", END_PORT_SHORT_OPT, END_PORT_LONG_OPT);
  fprintf (stdout, "\n");
  fprintf (stdout, "Extra options:\n");
  fprintf (stdout, "      %s, %s       quiet mode (no console output, logs to file)\n", QUIET_MODE_SHORT_OPT, QUIET_MODE_LONG_OPT);
  fprintf (stdout, "      %s, %s   log scan results to file (%s)\n", ENABLE_LOGFILE_SHORT_OPT, ENABLE_LOGFILE_LONG_OPT, KNOCKER_LOG_FILE);
  fprintf (stdout, "      %s, %s        disable colored output\n", NO_COLORS_SHORT_OPT, NO_COLORS_LONG_OPT);
  fprintf (stdout, "\n");
  fprintf (stdout, "Info options:\n");
  fprintf (stdout, "      %s,  %s             display this help and exit\n", HELP_SHORT_OPT, HELP_LONG_OPT);
  fprintf (stdout, "      %s,  %s          output version information and exit\n", VERSION_SHORT_OPT, VERSION_LONG_OPT);
  fprintf (stdout, "\n");
  fprintf (stdout, "SEE THE MAN PAGE FOR MORE DESCRIPTIONS, AND EXAMPLES\n");
  fprintf (stdout, "Report bugs to %s.\n", knocker_core_author_email);
}


/*
   ============================================================================
   ============================================================================
*/
int  knocker_args_init  (knocker_args_t *args, int logfile, int quiet, int colors)
{
   args->hname   = NULL;
   args->hip     = NULL;
   args->port    = 0;
   args->sport   = 0;
   args->eport   = 0;
   args->logfile = logfile;
   args->quiet   = quiet;
   args->colors  = colors;

   return 0;
}


/*
   ============================================================================
   ============================================================================
*/
void knocker_args_free  (knocker_args_t *args)
{
   if ( args->hname != NULL)
     free(args->hname);

   if (args->hip != NULL)
     free(args->hip);
}


/*
   ============================================================================
   ============================================================================
*/
/*
int  knocker_args_argument (knocker_args_t *args, unsigned short argument)
{

}
*/

/*
   ============================================================================
   ============================================================================
*/
int knocker_args_parse (knocker_args_t *args, int argc, char *argv[])
{
  int i;

  if (argv[1] == NULL)
    {
      fprintf (stderr, "%s: no arguments given\n", PACKAGE);
      fprintf (stderr, "Try `%s %s' for more information.\n", PACKAGE, HELP_LONG_OPT);
      exit(EXIT_FAILURE);
    }

  for (i = 1; i <= argc - 1; i++)
    {
      if ((!strcmp (argv[i], VERSION_SHORT_OPT)) || (!strcmp (argv[i], VERSION_LONG_OPT)))
	{
	  print_version_info ();
	  exit(EXIT_SUCCESS);
	}
      else if ((!strcmp (argv[i], HELP_SHORT_OPT)) || (!strcmp (argv[i], HELP_LONG_OPT)))
	{
	  print_help_info ();
	  exit(EXIT_SUCCESS);
	}
      else if ((!strcmp (argv[i], NO_COLORS_SHORT_OPT)) || (!strcmp (argv[i], NO_COLORS_LONG_OPT)))
	{
	  /* Disable colored output */
	  args->colors = FALSE;
	}
      else if ((!strcmp (argv[i], ENABLE_LOGFILE_SHORT_OPT)) || (!strcmp (argv[i], ENABLE_LOGFILE_LONG_OPT)))
	{
	  /* log scan results to file */
	  args->logfile = TRUE;
	}
      else if ((!strcmp (argv[i], QUIET_MODE_SHORT_OPT)) || (!strcmp (argv[i], QUIET_MODE_LONG_OPT)))
	{
	  /* Enable quiet mode, disable stdout output, and log result to file */
	  args->quiet   = TRUE;
	  args->colors  = FALSE;
	  args->logfile = TRUE;
	}
      else if ((!strcmp (argv[i], HOST_SHORT_OPT)) || (!strcmp (argv[i], HOST_LONG_OPT)))
	{
	  if (++i == argc)
	    {
	      fprintf (stderr, "%s: error, no host to scan given\n", PACKAGE);
	      fprintf (stderr, "Try `%s %s' for more information.\n", PACKAGE, HELP_LONG_OPT);
	      knocker_args_free  (args);
	      exit (EXIT_FAILURE);
	    }

	  /* allocate space and set the hostname string arg*/
	  args->hname  = realloc ( args->hname, strlen(argv[i]) + 1);
	  strcpy (args->hname, argv[i]);
	}

      else if ((!strcmp (argv[i], SINGLE_PORT_SHORT_OPT)) || (!strcmp (argv[i], SINGLE_PORT_LONG_OPT)))
	{
	  if (++i == argc)
	    {
	      fprintf (stderr, "%s: error, no port number given\n", PACKAGE);
	      fprintf (stderr, "Try `%s %s' for more information.\n", PACKAGE, HELP_LONG_OPT);
	      knocker_args_free  (args);
	      exit (EXIT_FAILURE);
	    }
          /* set the single port number for the one-port-only-scan */
	  args->port = atoi (argv[i]);

	  if (!knocker_core_validate_port_number (args->port) )
	    {
	      fprintf (stderr, "%s: error, invalid port number `%d' given\n", PACKAGE, args->port);
	      knocker_args_free  (args);
	      exit (EXIT_FAILURE);
	    }
	}

      else if ((!strcmp (argv[i], START_PORT_SHORT_OPT)) || (!strcmp (argv[i], START_PORT_LONG_OPT)))
	{
	  if (++i == argc)
	    {
	      fprintf (stderr, "%s: error, no port number, where to start the scan from, given\n", PACKAGE);
	      fprintf (stderr, "Try `%s %s' for more information.\n", PACKAGE, HELP_LONG_OPT);
	      knocker_args_free  (args);
	      exit (EXIT_FAILURE);
	    }
          /* set the start port number */
	  args->sport = atoi (argv[i]);

	  if (!knocker_core_validate_port_number (args->sport))
	    {
	      fprintf (stderr, "%s: error, invalid start port number `%d' given\n", PACKAGE, args->sport);
	      knocker_args_free  (args);
	      exit (EXIT_FAILURE);
	    }
	}
      else if ((!strcmp (argv[i], END_PORT_SHORT_OPT)) || (!strcmp (argv[i], END_PORT_LONG_OPT)))
	{
	  if (++i == argc)
	    {
	      fprintf (stderr, "%s: error, no port number, where to scan to, given\n", PACKAGE);
	      fprintf (stderr, "Try `%s %s' for more information.\n", PACKAGE, HELP_LONG_OPT);
	      knocker_args_free  (args);
	      exit (EXIT_FAILURE);
	    }
          /* set the end port number */
	  args->eport = atoi (argv[i]);

	  if (!knocker_core_validate_port_number (args->eport))
	    {
	      fprintf (stderr, "%s: error, invalid end port number `%d' given\n", PACKAGE, args->eport);
	      knocker_args_free  (args);
	      exit (EXIT_FAILURE);
	    }
	}
      else
	{
	  fprintf (stderr, "%s: invalid arguments given\n", PACKAGE);
	  fprintf (stderr, "Try `%s %s' for more information.\n", PACKAGE, HELP_LONG_OPT);
	  knocker_args_free  (args);
	  exit (EXIT_FAILURE);
	}

    }				/* end of for loop */

  return 0;
}



